import java.awt.geom.*;
import java.awt.*;

/**
 * Clase encargada de manejar el comportamiento grafico del elastico
 */
public class GElastic extends Elastic implements Drawable {
	private static final double xPoints[]={0,1.0};
	private static final double yPoints[]={0,0};
	private static final Path2D.Double line = new Path2D.Double(Path2D.WIND_EVEN_ODD,xPoints.length);
	private Path2D.Double shape;
	private Stroke stroke;
	private  float stiffness;
	private Vector2D freeAendPosition;    // to draw a spring we need
	private Vector2D freeBendPosition;  // extremes well definied
	// when it is not connected.

	static {  // static initialization block. It creates a spring of length = 1.
		line.moveTo (xPoints[0], yPoints[0]);
		for (int index = 1; index < xPoints.length;index++)
			line.lineTo(xPoints[index], yPoints[index]);
	}
	/**
	 * Constructor de la clase 
	 * @param restLength  Largo en reposos del Elastico
	 * @param stiffness   Constante elastica del elastico 
	 */
	
	public GElastic(float restLength, float stiffness) {
		super(restLength, stiffness);
		freeAendPosition = new Vector2D(0,0);
		freeBendPosition = new Vector2D(restLength,restLength);
		AffineTransform at = AffineTransform.getTranslateInstance(0,0);
		Vector2D v = getVector();
		at.rotate(v.getX(), v.getY());
		at.scale(v.module(), restLength);
		shape = (Path2D.Double) at.createTransformedShape(line);
		stroke = new BasicStroke(1);
		this.stiffness=stiffness;
	}
	/**  Retorna la posicion A del Elastico
	 * @return Vector con  la posicion A del elastico
	 */
	public Vector2D getAendPosition() {
		if (a_end == null)
			return freeAendPosition;
		return super.getAendPosition();
	}
	/**  Retorna la posicion B del Elastico
	 * @return Vector con  la posicion B del elastico
	 */
	public Vector2D getBendPosition() {
		if (b_end == null)
			return freeBendPosition;
		return super.getBendPosition();
	}
	
	/** dibuja el objeto grafico 2D que se le pasa
	 * @param g  objeto Grafico 2D
	 */
	public void draw (Graphics2D g){
		Vector2D a=getAendPosition();
		double ax = a.getX();
		double ay = a.getY();
		Vector2D v = getVector();
		AffineTransform at = AffineTransform.getTranslateInstance(ax, ay);
		at.rotate(v.getX(), v.getY());
		at.scale(v.module(),  restLength);
		shape = (Path2D.Double) at.createTransformedShape(line);
		if (v.module() < restLength)
			g.setColor(Color.BLACK);
		else
			g.setColor(Color.RED);
	    g.setStroke(stroke);
	    g.draw(shape);
	    //se cambia de color para la constante elastica
	    g.setColor(Color.GREEN);
	    //Marcamos en el elastico el valor de la constante de elasticidad
	    g.drawString(String.valueOf(this.stiffness),(int)(getAendPosition().getX()+getBendPosition().getX())/2,(int)(getAendPosition().getY()+getBendPosition().getY())/2);
		
	}
	/** verifica que las coordenadas dadas coincidan con los bordes del elastico
	 * @param x posicion x
	 * @param y posicion y
	 * @return verdadero o falso si se cumple la condicion descrita anteriormente 
	 */
	public boolean contains(int x, int y){
		return shape.getBounds2D().contains(x,y);
	}
	/** mueve el objeto a la nueva posicion (x,y)
	 * @param x posicion x
	 * @param y posicion y
	 */
	public void dragTo(int x, int y){
		Vector2D a=getAendPosition();
		Vector2D b=getBendPosition();
		double da = Math.abs(a.getX()-x)+Math.abs(a.getY()-y); // "distance" to a
		double db = Math.abs(b.getX()-x)+Math.abs(b.getY()-y); // "distance" to a
		if (da < db) {
			detachAend();
			freeAendPosition.set(x,y);
		}
		else {
			detachBend();
			freeBendPosition.set(x,y);
		}
	}
	/** Actualiza el estado de las posiciones del elastico
	 * 
	 */
	public void updateState(){  // just to return to a rest length
		if (a_end==null && b_end==null) return;  // when it is released
		if (a_end!=null) {
			Vector2D v = getVector().unitary().times(restLength);
			freeBendPosition = getAendPosition().plus(v);
			return;
		}
		if (b_end!=null) {
			Vector2D v = getVector().unitary().times(restLength);
			freeAendPosition = getBendPosition().minus(v);
			return;
		}
	}
	
	/** crea un trazo de largo 3 de la clase {@link <a href="http://download.oracle.com/javase/6/docs/api/java/awt/BasicStroke.html">BasicStroke</a>}
	 * 
	 */
	public void setSelected() {
		stroke = new BasicStroke(3);
	}
	/** crea un trazo de largo 1 de la clase {@link <a href="http://download.oracle.com/javase/6/docs/api/java/awt/BasicStroke.html">BasicStroke</a>}
	 * 
	 */
	public void setReleased() {
		stroke = new BasicStroke(1);
	}

	/**
	    * Modifica el valor de la constante elastica del elastico.
	    * @param stiffness valor que tomara la constante de elasticidad del elastico
	    * 
	    */
	public void setStiffness(float stiffness){
		this.stiffness=stiffness;
	}
}